import numpy as np
from scipy.stats import norm
from sklearn.calibration import calibration_curve


def average_squared_calibration_error(
    targets: np.ndarray, probs: np.ndarray, n_bins: int = 10
) -> float:
    binned_ey, binned_probs = calibration_curve(
        targets, probs, n_bins=n_bins, strategy="quantile"
    )
    return float(np.mean((binned_ey - binned_probs) ** 2))


def average_smooth_squared_calibration_error(
    targets: np.ndarray, probs: np.ndarray, n_bins: int = 10, smoothness: float = 0.1
) -> float:
    assce = 0
    for i in range(1, n_bins + 1):
        kernels = norm.pdf(probs, i, smoothness)
        assce += np.mean(kernels) * np.mean(kernels * (targets - probs)) ** 2
    return assce



from typing import List, Optional, Tuple

import numpy as np

from caliber.binary_classification.base_smooth_fit_mixin import (
    BinaryClassificationSmoothFitMixin,
)


class BinaryClassificationLinearScalingSmoothFitMixin(
    BinaryClassificationSmoothFitMixin
):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if "bounds" not in self._minimize_options:
            self._minimize_options["bounds"] = self._get_bounds()

    def _get_x0(self) -> np.ndarray:
        return np.array([0.0, 1.0]) if self._has_intercept else np.array(1.0)

    def _get_bounds(self) -> List:
        return [(None, None), (0.0, None)] if self._has_intercept else [(0.0, None)]



from typing import Optional

from sklearn.metrics import log_loss

from caliber.binary_classification.linear_scaling.calibration.base import (
    CustomCalibrationBinaryClassificationLinearScaling,
)
from caliber.binary_classification.linear_scaling.smooth_fit_mixin_linear_scaling import (
    BinaryClassificationLinearScalingSmoothFitMixin,
)


class CrossEntropyBinaryClassificationLinearScaling(
    BinaryClassificationLinearScalingSmoothFitMixin,
    CustomCalibrationBinaryClassificationLinearScaling,
):
    def __init__(
        self, minimize_options: Optional[dict] = None, has_intercept: bool = True
    ):
        super().__init__(
            loss_fn=log_loss,
            minimize_options=minimize_options,
            has_intercept=has_intercept,
        )
